/*
 * SourceMod Radio
 *
 * Adds a radio player to SourceMod that supports direct station links or streaming audio
 *
 * Coded by dubbeh - www.dubbeh.net
 *
 * Licensed under the GPLv3
 *
 */


void GetRadioStationsFromFile(bool bParseCustomShoutCast)
{
	KeyValues kv = null;
	char szLocalPath[PLATFORM_MAX_PATH + 1];
	char szParentGenreName[64] = "";
	char szChildGenreName[64] = "";
	char szStationName[128] = "";
	int iParentGenreID = 0;
	int iParentGenreCount = 0;
	int iChildGenreID = 0;
	int iChildGenreCount = 0;
	bool bHasChildren = false;
	
	
	g_bStationsParsing = true;
	
	kv = new KeyValues("Radio Stations");
	
	if (bParseCustomShoutCast)
		BuildPath(Path_SM, szLocalPath, sizeof(szLocalPath), "configs/%s", g_szRadioStationsShoutCastCustomFile);
	else if (g_cVarRadioPlayType.IntValue == RADIO_PLAY_TYPE_DIRECT)
		BuildPath(Path_SM, szLocalPath, sizeof(szLocalPath), "configs/%s", g_szRadioStationsDirectBaseFile);
	else if (g_cVarRadioPlayType.IntValue == RADIO_PLAY_TYPE_VOLUME_CONTROL)
		BuildPath(Path_SM, szLocalPath, sizeof(szLocalPath), "configs/%s", g_szRadioStationsVolumeBaseFile);
	else if (g_cVarRadioPlayType.IntValue == RADIO_PLAY_TYPE_SHOUTCAST)
		BuildPath(Path_SM, szLocalPath, sizeof(szLocalPath), "configs/%s", g_szRadioStationsShoutCastBaseFile);
	
	LogMessage("[SM-RADIO] Importing KeyValues from file \"%s\"", szLocalPath);
	
	if (kv.ImportFromFile(szLocalPath)) {
		// This is incase we need to append Genre or Station ID's to the Key Values file
		// Keeps the index's persistant across server reboots when admins add new stations or genres without one.
		g_bWriteKVToDisk = false;
		
		kv.GetString("Off Page", g_szRadioOffPage, sizeof(g_szRadioOffPage), "about:blank");
		kv.GetString("Volume Wrapper", g_szVolumeWrapperURL, sizeof(g_szVolumeWrapperURL), "https://dubbeh.net/sm/rvw.html");
		kv.GetString("Shoutcast Player", g_szShoutCastURL, sizeof(g_szShoutCastURL), "https://radio.dubbeh.net/index.php");
		kv.GetString("Volume Track Query", g_szVolumeControlTrackQueryURL, sizeof(g_szVolumeControlTrackQueryURL), "https://radio.dubbeh.net/trackquery.php");
		
		if (bParseCustomShoutCast) {
			g_bCustUseGenres = view_as<bool>(kv.GetNum("Use Genres", 0));
			g_CustGenres = new CustomGenres();
			g_CustStations = new CustomStations();
		} else {
			// Put this here to make sure we don't overwrite the ShoutCast playlist version
			g_iShoutCastPlaylistVersion = kv.GetNum("ShoutCast Version", 0);
			if (g_cVarRadioPlayType.IntValue == RADIO_PLAY_TYPE_SHOUTCAST) {
				g_bUseGenres = true;
			} else {
				g_bUseGenres = view_as<bool>(kv.GetNum("Use Genres", 0));
			}
			g_Genres = new Genres();
			g_Stations = new Stations();
		}
		
		kv.GotoFirstSubKey(true);
		
		iParentGenreCount = 0;
		iChildGenreCount = 0;
		
		do {
			if ((g_bUseGenres && !bParseCustomShoutCast) || (g_bCustUseGenres && bParseCustomShoutCast)) {
				kv.GetSectionName(szParentGenreName, sizeof(szParentGenreName));
				
				if (szParentGenreName[0]) {
					iParentGenreID = kv.GetNum("Genre ID", 0);
					bHasChildren = ParseGenreEntry(kv, szParentGenreName, iParentGenreID, 0, bParseCustomShoutCast);
					
					kv.GotoFirstSubKey(true);
					
					if (bHasChildren) {
						// Run over the child genres
						do {
							kv.GetSectionName(szChildGenreName, sizeof(szChildGenreName));
							if (szChildGenreName[0]) {
								iChildGenreID = kv.GetNum("Genre ID", 0);
								ParseGenreEntry(kv, szChildGenreName, iChildGenreID, iParentGenreID, bParseCustomShoutCast);
								iChildGenreCount++;
								
								kv.GotoFirstSubKey(true);
								do {
									ParseStationEntry(kv, iChildGenreID, bParseCustomShoutCast);
								} while (kv.GotoNextKey(true));
								kv.GoBack();
							} else {
								LogMessage("[SM-RADIO] Child genre at index \"%d\" Has no name set. Please correct this error in the config file", iChildGenreCount + 1);
							}
						} while (kv.GotoNextKey(true));
						
						kv.GoBack();
					} else {
						// Current key has no children - Parse each base key as a station.
						do {
							ParseStationEntry(kv, iParentGenreID, bParseCustomShoutCast);
						} while (kv.GotoNextKey(true));
						
						kv.GoBack();
					}
					
					iParentGenreCount++;
					
				} else {
					LogMessage("[SM-RADIO] Parent genre at index \"%d\" Has no name set. Please correct this error in the config file", iParentGenreCount + 1);
				}
			} else {
				if (kv.GetSectionName(szStationName, sizeof(szStationName))) {
					ParseStationEntry(kv, 0, bParseCustomShoutCast);
				}
			}
		} while (kv.GotoNextKey(true));
		
		kv.GoBack();
		
		// Invalid Genre or Station ID's found - Need to update with fixed version
		if (g_bWriteKVToDisk) {
			kv.Rewind();
			kv.ExportToFile(szLocalPath);
			g_bWriteKVToDisk = false;
		}
	} else {
		LogMessage("[SM-RADIO] Error: unable to load \"%s\". Make sure you have the file in that location.", szLocalPath);
	}
	
	LogMessage("[SM-RADIO] Finishing parsing \"%s\" - Found \"%d\" parents and \"%d\" child genres", szLocalPath, iParentGenreCount, iChildGenreCount);
	g_bStationsParsing = false;
	delete kv;
}

bool ParseGenreEntry(KeyValues kv, char[] szGenreName, int iGenreID, int iParentGenreID, bool bIsCustomShoutCast)
{
	bool bHasChildren = false;
	
	if (iParentGenreID == 0)
		bHasChildren = view_as<bool>(kv.GetNum("Has Children", 0));
	else
		bHasChildren = false;
	
	// Does the current genre have an ID set?
	if (iGenreID == 0) {
		// Genre has no pre-defined index - need to write to disk
		iGenreID = GetURandomInt();
		kv.SetNum("Genre ID", iGenreID);
		g_bWriteKVToDisk = true;
	}
	
	if (bIsCustomShoutCast) {
		CustomGenre custgenre = CustomGenre();
		
		custgenre.m_iID = iGenreID;
		custgenre.m_iParentID = iParentGenreID;
		custgenre.SetName(szGenreName);
		custgenre.m_bHasChildren = bHasChildren;
		g_CustGenres.AddItem(custgenre);
	} else {
		Genre genre = Genre();
		
		genre.m_iID = iGenreID;
		genre.m_iParentID = iParentGenreID;
		genre.SetName(szGenreName);
		genre.m_bHasChildren = bHasChildren;
		g_Genres.AddItem(genre);
	}
	
	return bHasChildren;
}

void ParseStationEntry(KeyValues kv, int iGenreID, bool bCustomShoutCastStation)
{
	char szStationName[128] = "";
	char szStreamURL[128] = "";
	int iStationID = 0;
	
	kv.GetSectionName(szStationName, sizeof(szStationName));
	
	if (szStationName[0]) {
		iStationID = kv.GetNum("Station ID", 0);
		
		// Does the current Station have an ID set?
		if (iStationID == 0) {
			// Station has no pre-defined index - need to write to disk
			iStationID = GetURandomInt();
			kv.SetNum("Station ID", iStationID);
			g_bWriteKVToDisk = true;
		}
		
		if (g_cVarRadioPlayType.IntValue != RADIO_PLAY_TYPE_SHOUTCAST || bCustomShoutCastStation) {
			kv.GetString("Stream URL", szStreamURL, sizeof(szStreamURL), "");
			if (!szStreamURL[0]) {
				g_bWriteKVToDisk = false;
				LogMessage("[SM-RADIO] Station \"%s\" has no Stream URL set. Please correct this problem and reload map.", szStationName);
				return;
			}
		}
		
		if (bCustomShoutCastStation) {
			CustomStation custstation = CustomStation();
			
			custstation.m_iID = iStationID;
			custstation.SetName(szStationName);
			if (szStreamURL[0])
				custstation.SetStreamURL(szStreamURL);
			custstation.m_iGenreID = iGenreID;
			custstation.m_iLastUpdateTime = 0;
			g_CustStations.AddItem(custstation);
		} else {
			Station station = Station();
			
			station.m_iID = iStationID;
			station.SetName(szStationName);
			if (szStreamURL[0])
				station.SetStreamURL(szStreamURL);
			station.m_iGenreID = iGenreID;
			station.m_iLastUpdateTime = 0;
			g_Stations.AddItem(station);
		}
	} else {
		g_bWriteKVToDisk = false;
	}
}
